/*  Michael J. Chappell  September 7, 1992  Interactive ASCII Chart */
/*  asc.c     -- interactive ASCII and Extended character chart */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <dos.h>
#include <bios.h>
#include <screen.h>

#define ESC 27
#define TOPROW  4
#define TOPCOL  7
#define BOTROW 22
#define BOTCOL 73

#define BOX_COLOR       0x1f
#define CHAR_COLOR      0x1f
#define NORMAL_COLOR    0x17
#define HIGHLIGHT_COLOR 0x4f
#define CURSOR_COLOR    0x4f
#define BACKGROUND      0x01

void SetBorder(int c);
void ShowChart(void);
void ShowAscii(unsigned c, int attrib);
void MoveCursor(unsigned c);
void ShowValues(unsigned c);
int  RowAsc(unsigned c);
int  ColAsc(unsigned c);
void DisplayBox(int c, toprow, topcol, botrow, botcol); /* c is attribute */
void pokescr(unsigned address, value);
void beep(void);

int oldc, video, row, col;
unsigned vseg = NULL;
int mode;
char ascdata[256][80];

int main(int argc, char **argv)
{
	unsigned temp, i, c = 1;
	unsigned key;
	FILE *screen;
	FILE *FP;
	char fullpath[80];
	char *temp_ptr;

	strcpy(fullpath, argv[0]);
	if (temp_ptr = strrchr(fullpath, '\\')) {
		  *(++temp_ptr) = '\0';
		  strcat(fullpath, "asc.dat");
	} else strcpy(fullpath, "asc.dat");
	for (i=0; i<256; i++) strcpy(ascdata[i], "ASC.DAT NOT FOUND");
	if (FP=fopen(fullpath, "r")) {
		  for (i=0; i<256; i++)
			   fscanf(FP, "%[^\n]%*c", ascdata[i]);
		  fclose(FP);
	}

	row = cursrow(); col = curscol();
	screen = savescreen();
	mode = getvmode();
	if (mode == 7) vseg = 0xB000; else vseg = 0xB800;
	if (mode == 7) setvmode(7); else setvmode(3);
	cursoff();
	ShowChart();
	MoveCursor(c);
	while ((key = getch()) != ESC) {
		if (key == NULL) continue;
		temp = c;
		switch (key) {
			case 'H': c -= 32; break;
			case 'P': c += 32; break;
			case 'K': c--; break;
			case 'M': c++; break;
		} /* end of switch (key) */
		if ((RowAsc(temp)!=RowAsc(c))&&(ColAsc(temp)>ColAsc(c))) c-=32;
		if ((RowAsc(temp)!=RowAsc(c))&&(ColAsc(temp)<ColAsc(c))) c+=32;
		c %= 256;
		MoveCursor(c);
	} /* end while loop */
	curson();
	loadscreen(screen);
	poscurs(row, col);
}

void ShowChart(void)
{
	unsigned c = 0;

	SetBorder(BACKGROUND);
	clrscrn2(NORMAL_COLOR);

	DisplayBox((mode==7)?15:BOX_COLOR, TOPROW, TOPCOL, BOTROW, BOTCOL);
	for (c = 0; c <= 255; c++) ShowAscii(c, NORMAL_COLOR);
	poscurs(TOPROW - 3, TOPCOL + 44);
	printf("DEC OCT  HEX   BINARY");
	poscurs(TOPROW - 2, TOPCOL);
	printf("CURSOR KEYS MOVE; <ESC> QUITS");
	poscurs(BOTROW + 1, TOPCOL + 1);
	printf(" INTERACTIVE ASCII CHART  BY:MICHAEL J. CHAPPELL SEPTEMBER 4, 1994");
}

void ShowAscii(unsigned c, int attrib)
{
	int atr, temp;
	
	 atr = attrib<<8;
	 temp = RowAsc(c)*160+ColAsc(c)*2;
	 pokescr(temp-162, atr+32);
	 pokescr(temp-160, atr+32);
	 pokescr(temp-158, atr+32);
	 pokescr(temp-002, atr+32);
	 pokescr(temp+002, atr+32);
	 pokescr(temp+158, atr+32);
	 pokescr(temp+160, atr+32);
	 pokescr(temp+162, atr+32);
	 pokescr(RowAsc(c)*160+ColAsc(c)*2, attrib*256+c);
}

void ShowCursor(unsigned c, int attrib)
{
	int temp;
	unsigned atr;

	atr = attrib<<8;
	temp = RowAsc(c)*160+ColAsc(c)*2;
	if (atr) {
		pokescr(temp-162, atr+218);
		pokescr(temp-160, atr+196);
		pokescr(temp-158, atr+191);
		pokescr(temp-002, atr+179);
		pokescr(temp+002, atr+179);
		pokescr(temp+158, atr+192);
		pokescr(temp+160, atr+196);
		pokescr(temp+162, atr+217);
	} else {
		pokescr(temp-162, 1824);
		pokescr(temp-160, 1824);
		pokescr(temp-158, 1824);
		pokescr(temp-002, 1824);
		pokescr(temp+002, 1824);
		pokescr(temp+158, 1824);
		pokescr(temp+160, 1824);
		pokescr(temp+162, 1824);
	}
}

int RowAsc(unsigned c)
{
	return(2+TOPROW+(c/32)*2);
}

int ColAsc(unsigned c)
{
	return(2+TOPCOL+(c%32)*2);
}

void MoveCursor(unsigned c)
{
	if (c == oldc) { beep(); return; }

	ShowCursor(oldc, 0);
	ShowAscii(oldc, NORMAL_COLOR);
	ShowAscii(c, HIGHLIGHT_COLOR);
	ShowCursor(c, CURSOR_COLOR);
	oldc = c;
	ShowValues(c);
}

void ShowValues(unsigned c)
{
	char bin[9];
	unsigned t;
	int i;

	t = c; bin[8] = '\0';
	for (i=0; i<8; i++) { bin[i] = ((t & 128)?'1':'0'); t<<=1; }

	poscurs(TOPROW - 2, TOPCOL + 35);
	printf("                              ");
	poscurs(TOPROW - 2, TOPCOL + 35);
	printf(" ASCII = %3d 0%-3o 0x%02X %s", c, c, c, bin);
	pokescr((TOPROW - 2)*160+(TOPCOL + 32)*2, CHAR_COLOR*256+c);
	poscurs(TOPROW - 1, TOPCOL);
	printf(
	  "                                                                    ");
	poscurs(TOPROW - 1, TOPCOL);
	printf("%s", ascdata[c]);
}

void pokescr(unsigned address, value)
{
	poke(vseg, address, value);
}

void DisplayBox(int c, toprow, topcol, botrow, botcol) /* c is attribute */
{
	int i;

	topcol *= 2;
	botcol *= 2;
	toprow *= 160;
	botrow *= 160;

	for (i = toprow+topcol+2;i<toprow+botcol;i += 2) pokescr(i, c*256+205);
	for (i = toprow+botcol+160;i<botrow+botcol;i += 160) pokescr(i, c*256+179);
	for (i = botrow+botcol-2;i>botrow+topcol;i -= 2) pokescr(i, c*256+205);
	for (i = botrow+topcol-160;i>toprow+topcol;i -= 160) pokescr(i, c*256+179);
	pokescr(toprow+topcol, c*256+213);
	pokescr(toprow+botcol, c*256+184);
	pokescr(botrow+botcol, c*256+190);
	pokescr(botrow+topcol, c*256+212);
}

void beep(void)
{
	sound(800, 3);
}

void SetBorder(int c) {
	union REGS r;
	
	r.h.ah = 0x0b;
	r.x.bx = c;
	int86(0x0b, &r, &r);
} /* SetBorder */
